/*
 *  Terminal IIIMCF Interface/Terminal IIIMCF Input Method
 *  Copyright (c) 2005 Young Chol Song <nskystars at yahoo.com>
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer
 *     in the documentation and/or other materials provided with the
 *     distribution.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY
 *  EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE
 *  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 *  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 *  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 *  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 *  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * To compile test code:
 *  gcc -Wall -DHAVE_STDINT_H -I/usr/include/iiim -liiimcf -lcurses -o test test.c tii.c
 */


#include <stdio.h>
#include <stdlib.h>
#include <iiimcf.h>
#include "tii.h"

int
main(void)
{
	int i, j, trigger_auto;
	char ** im_list, ** lang_list;
	char test_ch;

	IIIMCF_handle handle = NULL;
	IIIMCF_context context = NULL;

	if(!tii_init_iiimcf()) goto exit;

	handle = tii_create_handle();
	if(handle == NULL) goto exit;

	/* for Korean (hangul) */
	IIIMCF_input_method im = tii_find_im(handle, "hangul");
	IIIMCF_language lang = tii_find_lang(handle, "hangul", "ko");

	/* for Chinese (newpy) */
/*	IIIMCF_input_method im = tii_find_im(handle, "newpy");
	IIIMCF_language lang = tii_find_lang(handle, "newpy", "zh_CN");
*/
	/* for Japanese (CannaLE, need cannaserver) */
/*	IIIMCF_input_method im = tii_find_im(handle, "CannaLE");
	IIIMCF_language lang = tii_find_lang(handle, "CannaLE", "ja");
*/
	if(im == NULL || lang == NULL) goto exit;

	trigger_auto = -1;
	context = tii_create_context(handle, im, lang, trigger_auto);
	if(context == NULL) goto exit;

	fprintf(stderr, "IIIMCF initialization success\n");

	im_list = tii_get_im_list(handle);
	for(i = 0; im_list[i] != NULL; i++) {
		fprintf(stderr, "[message] Input method %s\n", im_list[i]);
		lang_list = tii_get_lang_list(tii_find_im(handle, im_list[i]));
		fprintf(stderr, "[message] Languages");
		for(j = 0; lang_list[j] != NULL; j++) {
			fprintf(stderr, ", %s", lang_list[j]);
		}
		fprintf(stderr, "\n");
		tii_free_list(lang_list);
	}
	tii_free_list(im_list);

	test_ch = 0;
	if(!tii_fwd_trigger(context, 1)) goto exit;

	fprintf(stderr, "Type Ctrl-D to exit. '!' toggles input method.\n");

	test_ch = getchar();
	while(test_ch >= 0) {
		if(test_ch == '!') {
			if(tii_get_conversion_mode(context) == 1)
				tii_fwd_trigger(context, 0);
			else
				tii_fwd_trigger(context, 1);
			if(!tii_process_events(context)) goto exit;
		}
		else {
			if(!tii_fwd_key(context, 0, test_ch, 0))
				fprintf(stderr, "Commit %d\n", test_ch);
			if(!tii_process_events(context)) goto exit;
		}
		test_ch = getchar();
	}

exit:
	if(context != NULL)
		tii_destroy_context(context);
	if(handle != NULL)
		tii_destroy_handle(handle);
	tii_finalize_iiimcf();

	return 0;
}
