/*
 *  Terminal IIIMCF Interface/Terminal IIIMCF Input Method
 *  Copyright (c) 2005 Young Chol Song <nskystars at yahoo.com>
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer
 *     in the documentation and/or other materials provided with the
 *     distribution.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY
 *  EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE
 *  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 *  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 *  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 *  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 *  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


#define DEBUG 0

int wchar_to_utf8(wchar_t c, char * outbuf, int bufsize);
char * card16_to_utf8(const IIIMP_card16 * pu16);
char * iiimcf_text_to_utf8(IIIMCF_text t);

void tii_register_error_handler(void (*handler_func)(char *, int));
void tii_register_message_handler(void (*handler_func)(char *, char *));
void tii_register_lookup_handler(void (*handler_func)(char *, int, int, int, int, int, int, char **, char **));
void tii_register_preedit_handler(void (*handler_func)(char *, int));
void tii_register_commit_handler(void (*handler_func)(char *));
void tii_register_status_handler(void (*handler_func)(char *));
void tii_register_keyevent_handler(void (*handler_func)(int, char, int));

int tii_fwd_trigger(IIIMCF_context cx, int flag);
int tii_fwd_key(IIIMCF_context cx, int keycode, int keychar, int mod);

int tii_process_events(IIIMCF_context cx);

char ** tii_get_im_list(IIIMCF_handle hx);
char ** tii_get_lang_list(IIIMCF_input_method im);
IIIMCF_input_method tii_find_im(IIIMCF_handle hx, char * im_str);
IIIMCF_language tii_find_lang(IIIMCF_handle hx, char * im_str, char * lang_str);

void tii_free_list(char ** im_lang_arr);

int tii_get_conversion_mode(IIIMCF_context cx);

int tii_init_iiimcf(void);
void tii_finalize_iiimcf(void);

IIIMCF_handle tii_create_handle(void);
void tii_destroy_handle(IIIMCF_handle hx);

IIIMCF_context tii_create_context(IIIMCF_handle hx, IIIMCF_input_method im, IIIMCF_language lang, int trigger_auto);
void tii_destroy_context(IIIMCF_context cx);
